/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCSSValue.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/PrivateGdomCSSValue.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateIntGdomCSSValue.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMCSSValue::GDOMCSSValue(PassRefPtr<WebCore::CSSValue> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMCSSValue::~GDOMCSSValue()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "CSSValue.h"
#include <gdom/GdomCSSValue.h>
#include <gdom/PrivateGdomCSSValue.h>
#include <gdom/PrivateIntGdomCSSValue.h>
#include <gdom/GdomCSSValueprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomCSSValue, gdom_css_value, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomCSSValuePrivate {
    class WebCore::CSSValue *item;
};

#define GDOM_CSS_VALUE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_CSS_VALUE, GdomCSSValuePrivate))
*/
G_END_DECLS
WebCore::CSSValue *
gdom_css_value_get_private(GdomCSSValue *request)
{
    GDOMCSSValue *res;
    res = static_cast<GDOMCSSValue*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::CSSValue*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_CSS_TEXT,
    PROP_CSS_VALUE_TYPE,
};


static void gdom_css_value_finalize(GObject* object)
{
    /*GdomCSSValue* request = GDOM_CSS_VALUE(object);*/
    /*GdomCSSValuePrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_css_value_parent_class)->finalize(object);
}

static void gdom_css_value__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomCSSValue *self = GDOM_CSS_VALUE(object);
    WebCore::CSSValue * gself = (gdom_css_value_get_private(self));
    switch (prop_id) {
     case PROP_CSS_TEXT:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setCssText(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_css_value__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomCSSValue *self = GDOM_CSS_VALUE(object);
    WebCore::CSSValue * gself = (gdom_css_value_get_private(self));
    switch (prop_id) {
    case PROP_CSS_TEXT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->cssText()));
         break;
    }
    case PROP_CSS_VALUE_TYPE:
    {
         g_value_set_uint (value, gself->cssValueType());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_css_value_class_init(GdomCSSValueClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_css_value_finalize;
    g_klass->set_property = gdom_css_value__set_property_fn;
    g_klass->get_property = gdom_css_value__get_property_fn;

    g_object_class_install_property (g_klass, PROP_CSS_TEXT,
            g_param_spec_string ("css_text", /* name */
                "css_value_css_text", /* short description */
                "read-write  gchar * CSSValue.css_text", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_CSS_VALUE_TYPE,
            g_param_spec_uint ("css_value_type", /* name */
                "css_value_css_value_type", /* short description */
                "read-only  gushort CSSValue.css_value_type", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXUINT16, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomCSSValuePrivate));*/
}

static void gdom_css_value_init(GdomCSSValue* request)
{
    /*GdomCSSValuePrivate* priv = GDOM_CSS_VALUE_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

